pub mod server;
pub mod user;

use clap::{Parser, Subcommand};
use figment::Figment;

use crate::config::Config;

use server::ServerCommand;
use user::UserCommand;

#[derive(Parser)]
#[clap(author, version, about, long_about = None)]
#[clap(propagate_version = true)]
pub struct NomiloCli {
    #[clap(subcommand)]
    pub command: Command
}

#[derive(Subcommand)]
pub enum Command {
    /// Manage web server
    #[clap(subcommand)]
    Server(ServerCommand),
    /// Manage users
    #[clap(subcommand)]
    User(UserCommand)
}


pub trait NomiloCommand {
    fn run(self, figment: Figment, app_config: Config);
}

impl NomiloCommand for NomiloCli {
    fn run(self, figment: Figment, app_config: Config) {
        match self.command {
            Command::Server(sub) => sub.run(figment, app_config),
            Command::User(sub) => sub.run(figment, app_config),
        };
    }
}
