use rocket::http::{Cookie, SameSite, CookieJar};
use rocket::State;

use crate::config::Config;
use crate::DbConn;
use crate::models;


pub async fn do_login(
    conn: DbConn,
    config: &State<Config>,
    auth_request: models::AuthTokenRequest,
    cookies: &CookieJar<'_>
) -> Result<models::Session, models::UserError> {
	let session_duration = config.web_app.token_duration;

    let session = conn.run(move |c| {
        let user_info = models::LocalUser::get_user_by_creds(
            c,
            &auth_request.username,
            &auth_request.password
        )?;

        models::Session::new(c, &user_info, session_duration)
    }).await?;

    // Conversion between different date / time libraries, very cursed, I don't like that
    // About unwrap: I guess too bad if session time is over year 9999 (current max time if time-rs)
    let expires = time::OffsetDateTime::from_unix_timestamp(session.expires_at.timestamp()).unwrap();

    let session_cookie = Cookie::build(models::session::COOKIE_NAME, session.session_id.clone())
        .same_site(SameSite::Strict)
        .secure(true)
        .http_only(true)
        .expires(expires)
        .finish();

    cookies.add(session_cookie);

    Ok(session)
}
