use crate::dns;


// TODO: Use model types instead of dns types as input / output and only convert internaly?

pub type ConnectorResult<T> = Result<T, Box<dyn ConnectorError>>;

// Zone content api
// E.g.: DNS update + axfr, zone file read + write
#[async_trait]
pub trait RecordConnector: Send {
	async fn get_records(&mut self, zone: dns::Name, class: dns::DNSClass) -> ConnectorResult<Vec<dns::Record>>;
	async fn add_records(&mut self, zone: dns::Name, class: dns::DNSClass, new_records: Vec<dns::Record>) -> ConnectorResult<()>;
	async fn update_records(&mut self, zone: dns::Name, class: dns::DNSClass, old_records: Vec<dns::Record>, new_records: Vec<dns::Record>) -> ConnectorResult<()>;
	async fn delete_records(&mut self, zone: dns::Name, class: dns::DNSClass, records: Vec<dns::Record>) -> ConnectorResult<()>;
	// delete_records
}

// Zone management api, todo
// E.g.: Manage catalog zone, dynamically generate knot / bind / nsd config...
#[async_trait]
pub trait ZoneConnector: Send {
	// get_zones
	// add_zone
	// delete_zone
	async fn zone_exists(&mut self, zone: dns::Name, class: dns::DNSClass) -> ConnectorResult<()>;
}

pub trait ConnectorError: std::fmt::Debug + std::fmt::Display {
	fn is_proto_error(&self) -> bool;
	fn zone_name(&self) -> Option<dns::Name>;
}
