pub mod client;
pub mod dns_connector;
pub mod connector;

// Reexport trust dns types for convenience
pub use trust_dns_client::rr::rdata::{
    DNSSECRData, caa, sshfp, mx, null, soa, srv, txt
};
pub use trust_dns_client::rr::{
    RData, DNSClass, Record
};
pub use trust_dns_proto::rr::Name;
pub use trust_dns_proto::rr::dnssec::rdata::tsig::TsigAlgorithm;

// Reexport module types
pub use connector::{RecordConnector, ZoneConnector, ConnectorError};
pub use dns_connector::{DnsConnectorClient, DnsConnectorError};
pub use client::DnsClient;


use rocket::{Request, State, http::Status, request::{FromRequest, Outcome}};
use rocket::outcome::try_outcome;

use crate::config::Config;


#[rocket::async_trait]
impl<'r> FromRequest<'r> for Box<dyn RecordConnector> {
    type Error = ();
    async fn from_request(request: &'r Request<'_>) -> Outcome<Self, Self::Error> {
        let config = try_outcome!(request.guard::<&State<Config>>().await);
        match DnsClient::from_config(&config.dns).await {
            Err(e) => {
                error!("Failed to connect to DNS server: {}", e);
                Outcome::Failure((Status::InternalServerError, ()))
            },
            Ok(c) => Outcome::Success(Box::new(DnsConnectorClient::new(c)))
        }
    }
}

#[rocket::async_trait]
impl<'r> FromRequest<'r> for Box<dyn ZoneConnector> {
    type Error = ();
    async fn from_request(request: &'r Request<'_>) -> Outcome<Self, Self::Error> {
        let config = try_outcome!(request.guard::<&State<Config>>().await);
        match DnsClient::from_config(&config.dns).await {
            Err(e) => {
                error!("Failed to connect to DNS server: {}", e);
                Outcome::Failure((Status::InternalServerError, ()))
            },
            Ok(c) => Outcome::Success(Box::new(DnsConnectorClient::new(c)))
        }
    }
}
