use rocket::http::CookieJar;
use rocket::serde::json::Json;
use rocket::State;
use rocket::http::Status;

use crate::config::Config;
use crate::DbConn;
use crate::models;

use crate::controllers::do_login;


#[post("/users/me/token", data = "<auth_request>")]
pub async fn create_auth_token(
    conn: DbConn,
    config: &State<Config>,
    auth_request: Json<models::AuthTokenRequest>,
    cookies: &CookieJar<'_>
) -> Result<Json<models::Session>, models::ErrorResponse> {

    do_login(
        conn,
        config,
        auth_request.into_inner(),
        cookies
    ).await.map(|s| Json(s)).map_err(models::ErrorResponse::from)
}

#[post("/users", data = "<user_request>")]
pub async fn create_user<'r>(
    conn: DbConn,
    user_request: Json<models::CreateUserRequest>,
    user_info: Result<models::UserInfo, models::ErrorResponse>,
) -> Result<Status, models::ErrorResponse> {
    let user_request = user_request.into_inner();

    if user_request.role.is_some() && !user_info?.is_admin() {
        return models::ErrorResponse::new(
                Status::Forbidden,
                "Only user with admin role can create a user with a specific role".into()
            ).err()
    }

    conn.run(|c| {
        models::LocalUser::create_user(&c, user_request)
    }).await?;

    Ok(Status::Created)
}
