use serde::Serialize;
use rocket::{State, Either};
use rocket::http::{Status, CookieJar};
use rocket::form::Form;
use rocket::response::Redirect;

use crate::template::Template;
use crate::config::Config;
use crate::models;
use crate::DbConn;
use crate::controllers::do_login;

#[derive(Serialize)]
pub struct LoginPage {
    error: Option<String>
}


#[get("/login")]
pub async fn get_login_page() -> Template<'static, LoginPage> {
    Template::new("pages/login.html", LoginPage { error: None })
}

#[post("/login", data = "<auth_request>")]
pub async fn post_login_page(
    conn: DbConn,
    config: &State<Config>,
    auth_request: Form<models::AuthTokenRequest>,
    cookies: &CookieJar<'_>
) -> Result<Redirect, Either<Template<'static, LoginPage>, Status>> {
    let res = do_login(
        conn,
        config,
        auth_request.into_inner(),
        cookies
    ).await;

    match res {
        Ok(_) => Ok(Redirect::to(uri!("/zones"))),
        Err(models::UserError::BadCreds) => Err(Either::Left(Template::new(
            "pages/login.html",
            LoginPage {
                error: Some("Identifants incorrects".to_string())
            }
        ))),
        Err(e) => Err(Either::Right(models::ErrorResponse::from(e).status))
    }
}
