use serde_json::{Value, json};
use serde::Serialize;
use rocket::http::{Status};
use rocket::http::uri::Origin;

use crate::template::Template;
use crate::models;
use crate::DbConn;


#[derive(Serialize)]
pub struct RecordsPage {
    zone: String
}


// TODO: Check if origin changes if application mounted on different path
#[get("/zone/<zone>/records")]
pub async fn get_zone_records_page(user_info: models::UserInfo, zone: models::AbsoluteName, conn: DbConn, origin: &Origin<'_>) -> Result<Template<'static, Value>, Status> {
    let zone_name = zone.to_utf8();

    let zones = conn.run(move |c| {
        if user_info.is_admin() {
            models::Zone::get_by_name(c, &zone_name)?;
            models::Zone::get_all(c)
        } else {
            user_info.get_zone(c, &zone_name)?;
            user_info.get_zones(c)

        }
    }).await.map_err(|e| models::ErrorResponse::from(e).status)?;


    Ok(Template::new(
        "pages/zone/records.html",
        json!({
            "current_zone": zone.to_utf8(),
            "zones": zones,
            "nav_page": origin.clone().into_normalized().path().as_str(),
            "nav_sections": vec!["zones", zone.to_utf8().as_str()],
        })
    ))
}

#[get("/zones")]
pub async fn get_zones_page(user_info: models::UserInfo, conn: DbConn, origin: &Origin<'_>) -> Result<Template<'static, Value>, Status> {
    let zones = conn.run(move |c| {
        if user_info.is_admin() {
            models::Zone::get_all(c)
        } else {
            user_info.get_zones(c)
        }
    }).await.map_err(|e| models::ErrorResponse::from(e).status)?;


    Ok(Template::new(
        "pages/zones.html",
        json!({
            "zones": zones,
            "nav_page": origin.clone().into_normalized().path().as_str(),
            "nav_sections": vec!["zones"],
        })
    ))
}
